#! /usr/bin/python

#
import time
import json
import argparse
#
import ctlab
import unic_helper


def get_string_from_result(text):
    pos = text.find("=")
    return text[pos + 1:]


def get_file_name_from_result(text):
    t1 = get_string_from_result(text)
    t2 = t1.replace("\"","")
    return t2


def get_float_value_from_result(text):
    pos = text.find("=")
    return float(text[pos + 1:])


def get_int_value_from_result(text):
    pos = text.find("=")
    return int(float(text[pos + 1:]))


def get_metadata(lab):
    config['META'] = [] 
    result = lab.send_command_result_str(lab.unic, "idn?")
    res = get_string_from_result(result)
    print("  254 --> \t",  res)
    config['META'].append({'index': 254, 'value': res, })
    
    result = lab.send_command_result_str(lab.unic, "98?")
    res = get_string_from_result(result)
    print("   98 --> \t", res)
    config['META'].append({'index': 98, 'value': res, })
    
    result = lab.send_command_result_str(lab.unic, "209?")
    res = get_int_value_from_result(result)
    print("  209 --> \t",  res)
    config['META'].append({'index': 209, 'value': hex(res), })


def get_configdata(lab):
    # 80
    # 200

    config['ADC_Offsets'] = [] 
    for idx in range(100, 108):  # this is stupid
        cmd = "%i?" % idx
        result = lab.send_command_result_str(lab.unic, cmd)
        res = get_int_value_from_result(result)
        print("%5d --> \t %s" % (idx, res))
        config['ADC_Offsets'].append({  
            'index': idx,
            'value': res,
        })
        time.sleep(0.1)

    config['ADC_Scale'] = [] 
    for idx in range(110, 118):  # this is stupid
        cmd = "%i?" % idx
        result = lab.send_command_result_str(lab.unic, cmd)
        res = get_float_value_from_result(result)
        print("%5d --> \t %s" % (idx, res))
        config['ADC_Scale'].append({  
            'index': idx,
            'value': res,
        })
        time.sleep(0.1)

    config['DAC_Offsets'] = [] 
    for idx in range(120, 124):  # this is stupid
        cmd = "%i?" % idx
        result = lab.send_command_result_str(lab.unic, cmd)
        res = get_int_value_from_result(result)
        print("%5d --> \t %s" % (idx, res));
        config['DAC_Offsets'].append({  
            'index': idx,
            'value': res,
        })
        time.sleep(0.1)

    config['DAC_Scale'] = [] 
    for idx in range(130, 134):  # this is stupid
        cmd = "%i?" % idx
        result = lab.send_command_result_str(lab.unic, cmd)
        res = get_float_value_from_result(result)
        print("%5d --> \t %s" % (idx, res))
        config['DAC_Scale'].append({  
            'index': idx,
            'value': res,
        })
        time.sleep(0.1)

    config['DAC_Base_Scale'] = [] 
    idx = 140
    cmd = str(idx) + "?" 
    result = lab.send_command_result_str(lab.unic, cmd)
    res = get_float_value_from_result(result)
    print("%5d --> \t %s" % (idx, res))
    config['DAC_Base_Scale'].append({  
        'index': idx,
        'value': res,
    })
    time.sleep(0.1)

    config['ADC_Base_Scale'] = [] 
    idx = 141
    cmd = str(idx) + "?" 
    result = lab.send_command_result_str(lab.unic, cmd)
    res = get_float_value_from_result(result)
    print("%5d --> \t %s" % (idx, res));
    config['ADC_Base_Scale'].append({  
        'index': idx,
        'value': res,
    })
    time.sleep(0.1)        

    config['DEFAULT_FPGA_CONFIG'] = [] 
    idx = 210
    cmd = str(idx) + "?" 
    result = lab.send_command_result_str(lab.unic, cmd)
    textres = get_file_name_from_result(result)
    print("%5d --> \t %s" % (idx, textres))
    config['DEFAULT_FPGA_CONFIG'].append({  
        'index': idx,
        'value': textres,
    })
    time.sleep(0.1)  

    config['DEFAULT_LOG_FILE'] = [] 
    idx = 211
    cmd = str(idx) + "?" 
    result = lab.send_command_result_str(lab.unic, cmd)
    textres = get_file_name_from_result(result)
    print("%5d --> \t %s" % (idx, textres))
    config['DEFAULT_LOG_FILE'].append({  
        'index': idx,
        'value': textres,
    })
    time.sleep(0.1)  


config = {}


def main():
    print("Download Config-data from Uni-C into JSON file")
    
    # From Config-File
    (serial_port, unic_config) = unic_helper.read_configfile('config.ini')
    # Parse the cmd-line
    parser = argparse.ArgumentParser(description='Dump the UNI-C Config data into a json file.', prefix_chars='-')
    parser.add_argument("-p", "--port", help="Used port number")
    parser.add_argument("-f", "--file", help="Filename config json file")
    args = parser.parse_args()
    # Use the old stuff, if cmd-line is empty
    if args.port is not None:
        serial_port = args.port
    if args.file is not None:
        unic_config = args.file
    # test
    print('port =', serial_port)
    print('file =', unic_config)

    # open the serial_port
    lab = ctlab.ctlab(serial_port)
    lab.check_devices(verbose=True)

    get_metadata(lab)
    get_configdata(lab)

    with open(unic_config, 'w') as outfile:  
        json.dump(config, outfile, indent=2)


if __name__ == "__main__":
    main()
